import traceback
from typing import Optional

from cpl_core.console import Console

from bot_api.abc.dto_abc import DtoABC
from bot_api.exception.service_error_code_enum import ServiceErrorCode


class ErrorDTO(DtoABC):
    def __init__(self, error_code: Optional[ServiceErrorCode], message: str):
        DtoABC.__init__(self)

        self._error_code = ServiceErrorCode.Unknown if error_code is None else error_code
        self._message = message

    @property
    def error_code(self) -> ServiceErrorCode:
        return self._error_code

    @property
    def message(self) -> str:
        return self._message

    def from_dict(self, values: dict):
        self._error_code = values["ErrorCode"]
        self._message = values["Message"]

    def to_dict(self) -> dict:
        return {"errorCode": int(self._error_code.value), "message": self._message}
