from datetime import datetime

from cpl_core.database import TableABC

from bot_data.model.user import User


class UserMessageCountPerHour(TableABC):
    def __init__(
        self,
        date: str,
        hour: int,
        xp_count: int,
        user: User,
        created_at: datetime = None,
        modified_at: datetime = None,
        id=0,
    ):
        self._id = id
        self._date = date
        self._hour = hour
        self._xp_count = xp_count
        self._user = user

        TableABC.__init__(self)
        self._created_at = created_at if created_at is not None else self._created_at
        self._modified_at = modified_at if modified_at is not None else self._modified_at

    @property
    def id(self) -> int:
        return self._id

    @property
    def date(self) -> datetime:
        return self._date

    @property
    def hour(self) -> int:
        return self._hour

    @property
    def xp_count(self) -> int:
        return self._xp_count

    @xp_count.setter
    def xp_count(self, value: int):
        self._modified_at = datetime.now().isoformat()
        self._xp_count = value

    @property
    def user(self) -> User:
        return self._user

    @staticmethod
    def get_select_all_string() -> str:
        return str(
            f"""
            SELECT * FROM `UserMessageCountPerHour`;
        """
        )

    @staticmethod
    def get_select_by_id_string(id: int) -> str:
        return str(
            f"""
            SELECT * FROM `UserMessageCountPerHour`
            WHERE `Id` = {id};
        """
        )

    @staticmethod
    def get_select_by_user_id_string(id: int) -> str:
        return str(
            f"""
            SELECT * FROM `UserMessageCountPerHour`
            WHERE `UserId` = {id};
        """
        )

    @staticmethod
    def get_select_by_user_id_and_date_string(id: int, date: datetime) -> str:
        date_str = f"{str(date.year).zfill(4)}-{str(date.month).zfill(2)}-{str(date.day).zfill(2)}%"

        return str(
            f"""
            SELECT * FROM `UserMessageCountPerHour`
            WHERE `UserId` = {id}
            AND `Date` LIKE '{date_str}'
            AND `Hour` = {date.hour};
        """
        )

    @property
    def insert_string(self) -> str:
        return str(
            f"""
            INSERT INTO `UserMessageCountPerHour` (
                `UserId`, `Date`, `Hour`, `XPCount`, `CreatedAt`, `LastModifiedAt`
            ) VALUES (
                {self._user.user_id},
                '{self._date}',
                {self._hour},
                {self._xp_count},
                '{self._created_at}',
                '{self._modified_at}'
            );
        """
        )

    @property
    def udpate_string(self) -> str:
        return str(
            f"""
            UPDATE `UserMessageCountPerHour`
            SET `XPCount` = '{self._xp_count}',
            `LastModifiedAt` = '{self._modified_at}'
            WHERE `Id` = {self._id};
        """
        )

    @property
    def delete_string(self) -> str:
        return str(
            f"""
            DELETE FROM `UserMessageCountPerHour`
            WHERE `Id` = {self._id};
        """
        )

    @staticmethod
    def delete_by_user_id_string(id: int) -> str:
        return str(
            f"""
            DELETE FROM `UserMessageCountPerHour`
            WHERE `UserId` = {id}
    """
        )
