from typing import Optional

from cpl_core.database.context import DatabaseContextABC
from cpl_query.extension import List

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.auto_role_repository_abc import AutoRoleRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.model.auto_role import AutoRole
from bot_data.model.auto_role_rule import AutoRoleRule


class AutoRoleRepositoryService(AutoRoleRepositoryABC):
    def __init__(self, logger: DatabaseLogger, db_context: DatabaseContextABC, servers: ServerRepositoryABC):
        self._logger = logger
        self._context = db_context
        self._servers = servers

        AutoRoleRepositoryABC.__init__(self)

    def get_auto_roles(self) -> List[AutoRole]:
        auto_roles = List(AutoRole)
        self._logger.trace(__name__, f"Send SQL command: {AutoRole.get_select_all_string()}")
        results = self._context.select(AutoRole.get_select_all_string())
        for result in results:
            auto_roles.append(
                AutoRole(
                    self._servers.get_server_by_id(result[1]), result[2], result[3], result[4], result[5], id=result[0]
                )
            )

        return auto_roles

    def get_auto_role_by_id(self, id: int) -> AutoRole:
        self._logger.trace(__name__, f"Send SQL command: {AutoRole.get_select_by_id_string(id)}")
        result = self._context.select(AutoRole.get_select_by_id_string(id))[0]
        return AutoRole(
            self._servers.get_server_by_id(result[1]), result[2], result[3], result[4], result[5], id=result[0]
        )

    def find_auto_role_by_id(self, id: int) -> Optional[AutoRole]:
        self._logger.trace(__name__, f"Send SQL command: {AutoRole.get_select_by_id_string(id)}")
        result = self._context.select(AutoRole.get_select_by_id_string(id))
        if result is None or len(result) == 0:
            return None

        result = result[0]

        return AutoRole(
            self._servers.get_server_by_id(result[1]), result[2], result[3], result[4], result[5], id=result[0]
        )

    def get_auto_roles_by_server_id(self, id: int) -> List[AutoRole]:
        self._logger.trace(__name__, f"Send SQL command: {AutoRole.get_select_by_server_id_string(id)}")
        auto_roles = List(AutoRole)
        results = self._context.select(AutoRole.get_select_by_server_id_string(id))
        for result in results:
            auto_roles.append(
                AutoRole(
                    self._servers.get_server_by_id(result[1]), result[2], result[3], result[4], result[5], id=result[0]
                )
            )

        return auto_roles

    def get_auto_role_by_message_id(self, id: int) -> AutoRole:
        self._logger.trace(
            __name__,
            f"Send SQL command: {AutoRole.get_select_by_message_id_string(id)}",
        )
        result = self._context.select(AutoRole.get_select_by_message_id_string(id))[0]
        return AutoRole(
            self._servers.get_server_by_id(result[1]), result[2], result[3], result[4], result[5], id=result[0]
        )

    def find_auto_role_by_message_id(self, id: int) -> Optional[AutoRole]:
        self._logger.trace(
            __name__,
            f"Send SQL command: {AutoRole.get_select_by_message_id_string(id)}",
        )
        result = self._context.select(AutoRole.get_select_by_message_id_string(id))
        if result is None or len(result) == 0:
            return None

        result = result[0]

        return AutoRole(
            self._servers.get_server_by_id(result[1]), result[2], result[3], result[4], result[5], id=result[0]
        )

    def add_auto_role(self, auto_role: AutoRole):
        self._logger.trace(__name__, f"Send SQL command: {auto_role.insert_string}")
        self._context.cursor.execute(auto_role.insert_string)

    def update_auto_role(self, auto_role: AutoRole):
        self._logger.trace(__name__, f"Send SQL command: {auto_role.udpate_string}")
        self._context.cursor.execute(auto_role.udpate_string)

    def delete_auto_role(self, auto_role: AutoRole):
        self._logger.trace(__name__, f"Send SQL command: {auto_role.delete_string}")
        self._context.cursor.execute(auto_role.delete_string)

    def get_auto_role_rules(self) -> List[AutoRoleRule]:
        auto_role_rules = List(AutoRoleRule)
        self._logger.trace(__name__, f"Send SQL command: {AutoRoleRule.get_select_all_string()}")
        results = self._context.select(AutoRoleRule.get_select_all_string())
        for result in results:
            auto_role_rules.append(AutoRoleRule(result[1], result[2], result[3], result[4], result[5], id=result[0]))

        return auto_role_rules

    def get_auto_role_rule_by_id(self, id: int) -> AutoRoleRule:
        self._logger.trace(__name__, f"Send SQL command: {AutoRoleRule.get_select_by_id_string(id)}")
        result = self._context.select(AutoRoleRule.get_select_by_id_string(id))[0]
        return AutoRoleRule(result[1], result[2], result[3], result[4], result[5], id=result[0])

    def get_auto_role_rules_by_auto_role_id(self, id: int) -> List[AutoRoleRule]:
        auto_role_rules = List(AutoRoleRule)
        self._logger.trace(
            __name__,
            f"Send SQL command: {AutoRoleRule.get_select_by_auto_role_id_string(id)}",
        )
        results = self._context.select(AutoRoleRule.get_select_by_auto_role_id_string(id))
        for result in results:
            auto_role_rules.append(AutoRoleRule(result[1], result[2], result[3], result[4], result[5], id=result[0]))

        return auto_role_rules

    def add_auto_role_rule(self, auto_role_rule: AutoRoleRule):
        self._logger.trace(__name__, f"Send SQL command: {auto_role_rule.insert_string}")
        self._context.cursor.execute(auto_role_rule.insert_string)

    def update_auto_role_rule(self, auto_role_rule: AutoRoleRule):
        self._logger.trace(__name__, f"Send SQL command: {auto_role_rule.udpate_string}")
        self._context.cursor.execute(auto_role_rule.udpate_string)

    def delete_auto_role_rule(self, auto_role_rule: AutoRoleRule):
        self._logger.trace(__name__, f"Send SQL command: {auto_role_rule.delete_string}")
        self._context.cursor.execute(auto_role_rule.delete_string)
