from typing import Callable

from ariadne import ObjectType
from cpl_query.extension import List

from bot_graphql.abc.filter_abc import FilterABC
from bot_graphql.filter.page import Page
from bot_graphql.filter.sort import Sort


class QueryABC(ObjectType):
    __abstract__ = True

    def __init__(self, name: str):
        ObjectType.__init__(self, name)

    def add_collection(self, name: str, get_collection: Callable, filter_type: type = None):
        def wrapper(*args, **kwargs):
            if filter_type is not None and "filter" in kwargs:
                kwargs["filter"] = FilterABC.get_collection_filter(filter_type, kwargs["filter"])
            else:
                kwargs["filter"] = None

            if "page" in kwargs:
                page = Page()
                page.from_dict(kwargs["page"])
                kwargs["page"] = page

            if "sort" in kwargs:
                sort = Sort()
                sort.from_dict(kwargs["sort"])
                kwargs["sort"] = sort

            return self._resolve_collection(get_collection(*args), *args, **kwargs)

        self.set_field(f"{name}s", wrapper)
        self.set_field(f"{name}_count", lambda *args: get_collection(*args).count())

    # @FilterABC.resolve_filter_annotation
    def _resolve_collection(self, collection: List, *_, filter: FilterABC = None, page: Page = None, sort: Sort = None):
        if filter is not None:
            return filter.filter(collection)

        if page is not None:
            return page.filter(collection)

        if sort is not None:
            return sort.filter(collection)

        return collection
