import os

from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceCollectionABC
from cpl_core.environment import ApplicationEnvironmentABC
from cpl_discord.discord_event_types_enum import DiscordEventTypesEnum
from cpl_discord.service.discord_collection_abc import DiscordCollectionABC

from bot_core.abc.module_abc import ModuleABC
from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from modules.level.command.level_group import LevelGroup
from modules.level.events.level_on_member_join_event import LevelOnMemberJoinEvent
from modules.level.events.level_on_message_event import LevelOnMessageEvent
from modules.level.events.level_on_voice_state_update_event import (
    LevelOnVoiceStateUpdateEvent,
)
from modules.level.level_seeder import LevelSeeder
from modules.level.service.level_service import LevelService


class LevelModule(ModuleABC):
    def __init__(self, dc: DiscordCollectionABC):
        ModuleABC.__init__(self, dc, FeatureFlagsEnum.level_module)

    def configure_configuration(self, config: ConfigurationABC, env: ApplicationEnvironmentABC):
        cwd = env.working_directory
        env.set_working_directory(os.path.dirname(os.path.realpath(__file__)))
        config.add_json_file(f"default-level.json", optional=False)
        env.set_working_directory(cwd)

    def configure_services(self, services: ServiceCollectionABC, env: ApplicationEnvironmentABC):
        services.add_transient(LevelSeeder)
        services.add_transient(LevelService)

        # commands
        self._dc.add_command(LevelGroup)

        # events
        self._dc.add_event(DiscordEventTypesEnum.on_message.value, LevelOnMessageEvent)
        self._dc.add_event(
            DiscordEventTypesEnum.on_voice_state_update.value,
            LevelOnVoiceStateUpdateEvent,
        )
        self._dc.add_event(DiscordEventTypesEnum.on_member_join.value, LevelOnMemberJoinEvent)
